package main

import (
	"fmt"
	"log"
	"time"

	"getvaultpw/internal/config"
	"getvaultpw/internal/protectString"
	"getvaultpw/internal/vault"
)

func main() {
	// initialize application configuration
	cfg := config.Init()

	defer func(cfg *config.Config) {
		if err := cfg.WriteConfig(cfg.ConfigFile); err != nil {
			log.Fatalf("[WARNING] Unable to update configuration file: %v", err)
		}
		log.Println("[DEBUG] shutdown sequence complete")
	}(cfg)

	// if we are passed a vault instance and a password we need to update the password for a environment
	if len(cfg.VaultInstance) != 0 && len(cfg.VaultPass) != 0 {
		cfg.ConfigFileData.VaultEnvironment[cfg.VaultInstance].EPass = protectString.Encrypt(cfg.VaultPass, cfg.ConfigFileData.Salt)
	}

	// get the password for a secret
	if len(cfg.SecretID) != 0 {
		for k, v := range cfg.ConfigFileData.Credential {
			if k == cfg.SecretID {
				if (v.TimeStamp+(60*60*2)) < time.Now().UTC().Unix() || len(cfg.ConfigFileData.Credential[k].Cache) == 0 {
					o, err := vault.GetCredential(cfg.ConfigFileData.VaultEnvironment[v.VaultEnv].Host,
						cfg.ConfigFileData.VaultEnvironment[v.VaultEnv].User,
						protectString.Decrypt(cfg.ConfigFileData.VaultEnvironment[v.VaultEnv].EPass, cfg.ConfigFileData.Salt),
						v.ID,
						v.Path)

					if err != nil {
						log.Fatalf("[ERROR] %v", err)
					}
					cfg.ConfigFileData.Credential[k].Cache = protectString.Encrypt(o, cfg.ConfigFileData.Salt)
					cfg.ConfigFileData.Credential[k].TimeStamp = time.Now().UTC().Unix()
					fmt.Println(o)
				} else {
					o := protectString.Decrypt(cfg.ConfigFileData.Credential[k].Cache, cfg.ConfigFileData.Salt)
					fmt.Println(o)
				}
			} else {
				log.Fatalf("[ERROR] Unable to find secret (%s) in config file: %s", cfg.SecretID, cfg.ConfigFile)
			}
		}
	}
}
